system ("jtagconfig --setparam 1 JtagClock 6M"); 
system ("jtagconfig --getparam 1 JtagClock"); 
my $bin_file = $ARGV[1];
my $PFL_SOF = "$ENV{AOCL_BOARD_PACKAGE_ROOT}/linux64/libexec/de5a_net_pfl.sof";

if ( $bin_file =~ m/\.bin$/i ) {
  -f $bin_file or print "Error: can't find $binfile file needed for flashing." and exit 1;
} else {
  print "Error: Currently only .bin files are accepted.\n";
  print "       Please use the .bin from the original compiled project\n";
  exit 1;
}

if (-e $PFL_SOF){
}else{
    printf "Error: can't find $PFL_SOF file needed for flashing.\nPress any key to continue...";
    <STDIN>;
 	  exit 1;
}


&flash_programming($bin_file);

exit;

############################################
# Menu to select target page 
############################################

sub	flash_programming
{
	my $bin_file =  $_[0];	
	my $sof_file = "flash.sof";	
	my $cof_file = "flash.cof";
	my $cdf_file = "flash.cdf";
	my $pof_file = "flash.pof";
	
	my $PAGE0_ADDR     = "00040000"; # hex
	my $PAGE0_END_ADDR = "02B3FFFF"; # hex
	my $PAGE1_ADDR     = "02B40000"; # hex 
	my $PAGE1_END_ADDR = "0563FFFF"; # hex 
	my $OPTION_OFFSET  = "00030000"; # hex 
	
	
	
	printf "========================= Page Selection =========================\n";
	printf "Please select the flash page where to store your FPGA configure data:\n";
	printf "[0] Factory Image Location(Address 0x$PAGE0_ADDR), SW3.4 = \"1\" (Down Position)\n";
	printf "[1] User Image Location(Address 0x$PAGE1_ADDR), SW3.4 = \"0\" (Up Position)\n";
	printf "Enter a digital number 0 or 1 (Or other values to exit the program) followed by pressing the \"Enter\" key:\n";


	$PAGE	=	<STDIN>;   
	chop($PAGE);


	if	(($PAGE eq "0") || ($PAGE eq "1")){
		printf "Flash Programming...\n";		
		
		# extract .sof from fpga.bin
		&extract_sof($bin_file, $sof_file);		
				
		# generate cof
		&gen_cof_file($cof_file, $pof_file, $sof_file, $PAGE0_ADDR, $PAGE0_END_ADDR, $PAGE1_ADDR, $PAGE1_END_ADDR, $OPTION_OFFSET);
		
		# generate cdf
		&gen_cdf_file($cdf_file, $pof_file, ($PAGE eq "0")?1:0, ($PAGE eq "1")?1:0);		
		
		# generate pof 
		system ("quartus_cpf --convert $cof_file");
		$? == 0  or die "Error: quartus_cpf failed";
		
		# start programming
		system ("quartus_pgm -c 1 $cdf_file");
		$? == 0  or die "Error: quartus_pgm failed";
		
		
		# delete temporal files
		unlink $sof_file;
		unlink $cof_file;
		unlink $cdf_file;
		unlink $pof_file;
		
	}		

} # END of flash_programming

#############################
# extract .sof from .bin
sub	extract_sof
{

	my $bin_file = $_[0];
	my $sof_file = $_[1];
	
	system ("aocl binedit $bin_file get .acl.sof $sof_file");
	$? == 0  or die "Error: aocl binedit failed";

} #extract_sof


#############################
# generate cof file
sub gen_cof_file
{
	my $cof_file       = $_[0];
	my $pof_file       = $_[1];
	my $sof_file       = $_[2];
	my $page0_addr     = $_[3];
	my $page0_end_addr = $_[4];
	my $page1_addr     = $_[5];
	my $page1_end_addr = $_[6];
	my $option_addr    = $_[7];

my $cof = <<END;
<?xml version="1.0" encoding="US-ASCII" standalone="yes"?>
<cof>
	<eprom_name>CFI_2GB</eprom_name>
	<output_filename>$pof_file</output_filename>
	<n_pages>2</n_pages>
	<width>1</width>
	<mode>12</mode>
	<sof_data>
		<start_address>$page1_addr</start_address>
		<end_address>$page1_end_addr</end_address>
		<user_name>Page_1</user_name>
		<page_flags>2</page_flags>
		<bit0>
			<sof_filename>$sof_file</sof_filename>
		</bit0>
	</sof_data>
	<sof_data>
		<start_address>$page0_addr</start_address>
		<end_address>$page0_end_addr</end_address>
		<user_name>Page_0</user_name>
		<page_flags>1</page_flags>
		<bit0>
			<sof_filename>$sof_file</sof_filename>
		</bit0>
	</sof_data>
	<version>10</version>
	<create_cvp_file>0</create_cvp_file>
	<create_hps_iocsr>0</create_hps_iocsr>
	<auto_create_rpd>0</auto_create_rpd>
	<rpd_little_endian>1</rpd_little_endian>
	<options>
		<map_file>1</map_file>
		<option_start_address>$option_addr</option_start_address>
		<dynamic_compression>0</dynamic_compression>
	</options>
	<advanced_options>
		<ignore_epcs_id_check>2</ignore_epcs_id_check>
		<ignore_condone_check>2</ignore_condone_check>
		<plc_adjustment>0</plc_adjustment>
		<post_chain_bitstream_pad_bytes>-1</post_chain_bitstream_pad_bytes>
		<post_device_bitstream_pad_bytes>-1</post_device_bitstream_pad_bytes>
		<bitslice_pre_padding>1</bitslice_pre_padding>
	</advanced_options>	
</cof>
END

	open COFFILE, ">$cof_file";
	print COFFILE $cof;
	close COFFILE;


} # gen_cof_file


#############################
# generate cdf file
sub gen_cdf_file
{
	my $cdf_file = $_[0];	
	my $pof_file = $_[1];	
	my $page0_on = $_[2];	
	my $page1_on = $_[3];	
	
	my $option_on = 0;  # turn off option bit
	
	my $board_path = $ENV{AOCL_BOARD_PACKAGE_ROOT};
	
	
my $cdf = <<CDFEND;
JedecChain;
	FileRevision(JESD32A);
	DefaultMfr(6E);

	P ActionCode(Ign)
		Device PartName(10AX115N3F45) Path("$board_path/linux64/libexec/") File("de5a_net_pfl.sof") MfrSpec(OpMask(1) SEC_Device(CFI_2GB) Child_OpMask(4 0 $page0_on $page1_on $option_on) PFLPath("$pof_file"));

ChainEnd;

AlteraBegin;
	ChainType(JTAG);
AlteraEnd;
CDFEND



	open CDFFILE, ">$cdf_file";
	print CDFFILE $cdf;
	close CDFFILE;

} # gen_cdf_file


